@@ -4,7 +4,7 @@ use strict;
 use warnings;
 use base qw(DateTime::Format::Natural::Compat);
 
-our $VERSION = '1.25';
+our $VERSION = '1.27';
 
 use constant MORNING   => '08';
 use constant AFTERNOON => '14';
@@ -286,6 +286,19 @@ sub _at
 {
     my $self = shift;
     $self->_register_trace;
+    $self->_at_time(@_);
+}
+
+sub _time
+{
+    my $self = shift;
+    $self->_register_trace;
+    $self->_at_time(@_);
+}
+
+sub _at_time
+{
+    my $self = shift;
     my $opts = pop;
     my $hours = $opts->{hours} || 0;
     my ($time) = @_;
@@ -308,12 +321,6 @@ sub _at
     }
 }
 
-sub _time
-{
-    my $self = shift;
-    $self->_at(@_);
-}
-
 sub _time_full
 {
     my $self = shift;
@@ -402,6 +409,25 @@ sub _final_weekday_in_month
     );
 }
 
+sub _first_last_day_unit
+{
+    my $self = shift;
+    $self->_register_trace;
+    my $opts = pop;
+    my ($year, $month, $day) = do {
+        @_ >= 3 ? @_ : (undef, @_);
+    };
+    $year ||= $self->{datetime}->year;
+    unless (defined $day) {
+        $day = $self->_Days_in_Month($year, $month);
+    }
+    $self->_set(
+        year  => $year,
+        month => $month,
+        day   => $day,
+    );
+}
+
 1;
 __END__
 
