@@ -8,7 +8,7 @@ use DateTime ();
 
 our ($VERSION, $Pure);
 
-$VERSION = '0.02';
+$VERSION = '0.04';
 
 BEGIN
 {
@@ -16,6 +16,7 @@ BEGIN
         Date::Calc->import(qw(
             Add_Delta_Days
             Day_of_Week
+            Days_in_Month
             Decode_Day_of_Week
             Decode_Month
             Nth_Weekday_of_Month_Year
@@ -56,6 +57,20 @@ sub _Day_of_Week
     }
 }
 
+sub _Days_in_Month
+{
+    my $self = shift;
+
+    if ($Pure) {
+        my ($year, $month) = @_;
+        my $dt = DateTime->last_day_of_month(year => $year, month => $month);
+        return $dt->day;
+    }
+    else {
+        return Days_in_Month(@_);
+    }
+}
+
 sub _Decode_Day_of_Week
 {
     my $self = shift;
@@ -111,9 +126,7 @@ sub _check_date
         local $@;
         eval {
             my $dt = $self->{datetime}->clone;
-            $dt->set_year($year);
-            $dt->set_month($month);
-            $dt->set_day($day);
+            $dt->set(year => $year, month => $month, day => $day);
         };
         return !$@;
     }
