@@ -9,7 +9,7 @@ use constant false => 0;
 
 use DateTime::Format::Natural::Helpers qw(%flag);
 
-our $VERSION = '1.23';
+our $VERSION = '1.26';
 
 our (%init,
      %timespan,
@@ -22,6 +22,7 @@ our (%init,
      @data_months_all,
      %data_conversion,
      %data_helpers,
+     %data_duration,
      %extended_checks,
      %grammar);
 
@@ -31,11 +32,11 @@ our (%init,
 %RE = (number    => qr/^(\d+)$/,
        year      => qr/^(\d{4})$/,
        time      => qr/^((?:\d{1,2})(?:\:\d{2})?)$/,
-       time_am   => qr/^((?:\d{1,2})(?:\:\d{2})?)(?:am)?$/,
-       time_pm   => qr/^((?:\d{1,2})(?:\:\d{2})?)pm$/,
+       time_am   => qr/^((?:\d{1,2})(?:\:\d{2})?)(?:am)?$/i,
+       time_pm   => qr/^((?:\d{1,2})(?:\:\d{2})?)pm$/i,
        time_full => qr/^(\d{1,2}\:\d{2}\:\d{2})$/,
-       day       => qr/^(\d+)(?:st|nd|rd|th)?$/,
-       monthday  => qr/^(\d{1,2})(?:st|nd|rd|th)?$/);
+       day       => qr/^(\d+)(?:st|nd|rd|th)?$/i,
+       monthday  => qr/^(\d{1,2})(?:st|nd|rd|th)?$/i);
 {
     my $i = 1;
 
@@ -82,6 +83,20 @@ our (%init,
         normalize   => sub { ${$_[0]} = ucfirst lc ${$_[0]} },
         abbreviated => sub { length ${$_[0]} == 3 },
     );
+
+    %data_duration = (
+        for => sub {
+            my ($date_strings) = @_;
+            return (scalar @$date_strings == 1
+                && $date_strings->[0] =~ /^for\s+/i);
+        },
+        first_last => sub {
+            my ($date_strings) = @_;
+            return (scalar @$date_strings == 2
+                && $date_strings->[0] =~ /^first$/i
+                && $date_strings->[1] =~ /^last\s+/i);
+        },
+    );
 }
 
 %extended_checks = (
@@ -263,7 +278,7 @@ our (%init,
              { 2 => [ $flag{noon_midnight} ] },
            ],
          ],
-         [ { unit => 'day' }, { hour => 12 } ],
+         [ { unit => 'day' }, {} ],
          [ '_unit_variant', '_daytime' ],
          {},
        ],
@@ -279,7 +294,7 @@ our (%init,
              { 2 => [ $flag{noon_midnight} ] },
            ],
          ],
-         [ { unit => 'day' }, { hour => 0 } ],
+         [ { unit => 'day' }, {} ],
          [ '_unit_variant', '_daytime' ],
          {},
        ],
@@ -295,7 +310,7 @@ our (%init,
              { 2 => [ $flag{noon_midnight} ] },
            ],
          ],
-         [ { unit => 'day' }, { hour => 12 } ],
+         [ { unit => 'day' }, {} ],
          [ '_unit_variant', '_daytime' ],
          {},
        ],
@@ -311,7 +326,7 @@ our (%init,
              { 2 => [ $flag{noon_midnight} ] },
            ],
          ],
-         [ { unit => 'day' }, { hour => 0 } ],
+         [ { unit => 'day' }, {} ],
          [ '_unit_variant', '_daytime' ],
          {},
        ],
@@ -327,7 +342,7 @@ our (%init,
              { 2 => [ $flag{noon_midnight} ] },
            ]
          ],
-         [ { unit => 'day' }, { hour => 12 } ],
+         [ { unit => 'day' }, {} ],
          [ '_unit_variant', '_daytime' ],
          {},
        ],
@@ -343,11 +358,116 @@ our (%init,
              { 2 => [ $flag{noon_midnight} ] },
            ],
          ],
-         [ { unit => 'day' }, { hour => 0 } ],
+         [ { unit => 'day' }, {} ],
          [ '_unit_variant', '_daytime' ],
          {},
        ],
     ],
+    daytime_variant_weekday => [
+       [ 'REGEXP', 'REGEXP', 'REGEXP' ],
+       [
+         { 0 => qr/^(noon)$/i, 1 => qr/^(next)$/i, 2 => $RE{weekday} },
+         [],
+         [],
+         [
+           [
+             { 0 => [ $flag{noon_midnight} ] },
+           ],
+           [
+             { 1 => [ $flag{last_this_next} ] },
+             { 2 => [ $flag{weekday_name}, $flag{weekday_num} ] },
+           ],
+         ],
+         [ {}, {} ],
+         [ '_daytime', '_count_day_variant_week' ],
+         {},
+       ],
+       [
+         { 0 => qr/^(midnight)$/i, 1 => qr/^(next)$/i, 2 => $RE{weekday} },
+         [],
+         [],
+         [
+           [
+             { 0 => [ $flag{noon_midnight} ] },
+           ],
+           [
+             { 1 => [ $flag{last_this_next} ] },
+             { 2 => [ $flag{weekday_name}, $flag{weekday_num} ] },
+           ],
+         ],
+         [ {}, {} ],
+         [ '_daytime', '_count_day_variant_week' ],
+         {},
+       ],
+       [
+         { 0 => qr/^(noon)$/i, 1 => qr/^(this)$/i, 2 => $RE{weekday} },
+         [],
+         [],
+         [
+           [
+             { 0 => [ $flag{noon_midnight} ] },
+           ],
+           [
+             { 1 => [ $flag{last_this_next} ] },
+             { 2 => [ $flag{weekday_name}, $flag{weekday_num} ] },
+           ],
+         ],
+         [ {}, {} ],
+         [ '_daytime', '_count_day_variant_week' ],
+         {},
+       ],
+       [
+         { 0 => qr/^(midnight)$/i, 1 => qr/^(this)$/i, 2 => $RE{weekday} },
+         [],
+         [],
+         [
+           [
+             { 0 => [ $flag{noon_midnight} ] },
+           ],
+           [
+             { 1 => [ $flag{last_this_next} ] },
+             { 2 => [ $flag{weekday_name}, $flag{weekday_num} ] },
+           ],
+         ],
+         [ {}, {} ],
+         [ '_daytime', '_count_day_variant_week' ],
+         {},
+       ],
+       [
+         { 0 => qr/^(noon)$/i, 1 => qr/^(last)$/i, 2 => $RE{weekday} },
+         [],
+         [],
+         [
+           [
+             { 0 => [ $flag{noon_midnight} ] },
+           ],
+           [
+             { 1 => [ $flag{last_this_next} ] },
+             { 2 => [ $flag{weekday_name}, $flag{weekday_num} ] },
+           ],
+         ],
+         [ {}, {} ],
+         [ '_daytime', '_count_day_variant_week' ],
+         {},
+       ],
+       [
+         { 0 => qr/^(midnight)$/i, 1 => qr/^(last)$/i, 2 => $RE{weekday} },
+         [],
+         [],
+         [
+           [
+             { 0 => [ $flag{noon_midnight} ] },
+           ],
+           [
+             { 1 => [ $flag{last_this_next} ] },
+             { 2 => [ $flag{weekday_name}, $flag{weekday_num} ] },
+           ],
+         ],
+         [ {}, {} ],
+         [ '_daytime', '_count_day_variant_week' ],
+         {},
+       ],
+    ],
     this_daytime => [
        [ 'SCALAR', 'REGEXP' ],
        [
@@ -550,7 +670,7 @@ our (%init,
            ],
          ],
          [ {}, { unit => 'day' } ],
-         [ '_time', '_unit_variant' ],
+         [ '_at', '_unit_variant' ],
          { truncate_to => 'minute' },
        ],
        [
@@ -564,7 +684,7 @@ our (%init,
            ],
          ],
          [ {}, { unit => 'day' } ],
-         [ '_time', '_unit_variant' ],
+         [ '_at', '_unit_variant' ],
          { truncate_to => 'minute' },
        ],
        [
@@ -578,7 +698,7 @@ our (%init,
            ],
          ],
          [ {}, { unit => 'day' } ],
-         [ '_time', '_unit_variant' ],
+         [ '_at', '_unit_variant' ],
          { truncate_to => 'minute' },
        ],
        [
@@ -624,6 +744,99 @@ our (%init,
          { truncate_to => 'minute' },
        ],
     ],
+    at_variant_weekday => [
+       [ 'REGEXP', 'REGEXP', 'REGEXP' ],
+       [
+         { 0 => $RE{time_am}, 1 => qr/^(next)$/i, 2 => $RE{weekday} },
+         [],
+         [],
+         [
+           [ 0 ],
+           [
+             { 1 => [ $flag{last_this_next} ] },
+             { 2 => [ $flag{weekday_name}, $flag{weekday_num} ] },
+           ],
+         ],
+         [ {}, {} ],
+         [ '_at', '_count_day_variant_week' ],
+         { truncate_to => 'minute' },
+       ],
+       [
+         { 0 => $RE{time_am}, 1 => qr/^(this)$/i, 2 => $RE{weekday} },
+         [],
+         [],
+         [
+           [ 0 ],
+           [
+             { 1 => [ $flag{last_this_next} ] },
+             { 2 => [ $flag{weekday_name}, $flag{weekday_num} ] },
+           ],
+         ],
+         [ {}, {} ],
+         [ '_at', '_count_day_variant_week' ],
+         { truncate_to => 'minute' },
+       ],
+       [
+         { 0 => $RE{time_am}, 1 => qr/^(last)$/i, 2 => $RE{weekday} },
+         [],
+         [],
+         [
+           [ 0 ],
+           [
+             { 1 => [ $flag{last_this_next} ] },
+             { 2 => [ $flag{weekday_name}, $flag{weekday_num} ] },
+           ],
+         ],
+         [ {}, {} ],
+         [ '_at', '_count_day_variant_week' ],
+         { truncate_to => 'minute' },
+       ],
+       [
+         { 0 => $RE{time_pm}, 1 => qr/^(next)$/i, 2 => $RE{weekday} },
+         [],
+         [],
+         [
+           [ 0 ],
+           [
+             { 1 => [ $flag{last_this_next} ] },
+             { 2 => [ $flag{weekday_name}, $flag{weekday_num} ] },
+           ],
+         ],
+         [ { hours => 12 }, {} ],
+         [ '_at', '_count_day_variant_week' ],
+         { truncate_to => 'minute' },
+       ],
+       [
+         { 0 => $RE{time_pm}, 1 => qr/^(this)$/i, 2 => $RE{weekday} },
+         [],
+         [],
+         [
+           [ 0 ],
+           [
+             { 1 => [ $flag{last_this_next} ] },
+             { 2 => [ $flag{weekday_name}, $flag{weekday_num} ] },
+           ],
+         ],
+         [ { hours => 12 }, {} ],
+         [ '_at', '_count_day_variant_week' ],
+         { truncate_to => 'minute' },
+       ],
+       [
+         { 0 => $RE{time_pm}, 1 => qr/^(last)$/i, 2 => $RE{weekday} },
+         [],
+         [],
+         [
+           [ 0 ],
+           [
+             { 1 => [ $flag{last_this_next} ] },
+             { 2 => [ $flag{weekday_name}, $flag{weekday_num} ] },
+           ],
+         ],
+         [ { hours => 12 }, {} ],
+         [ '_at', '_count_day_variant_week' ],
+         { truncate_to => 'minute' },
+       ],
+    ],
     month => [
        [ 'REGEXP' ],
        [
@@ -680,6 +893,39 @@ our (%init,
          },
        ]
     ],
+    month_day_at => [
+       [ 'REGEXP', 'REGEXP', 'REGEXP' ],
+       [
+         { 0 => $RE{month}, 1 => $RE{monthday}, 2 => $RE{time_am} },
+         [],
+         [],
+         [
+           [
+               1,
+             { 0 => [ $flag{month_name}, $flag{month_num} ] },
+           ],
+           [ 2 ],
+         ],
+         [ {}, {} ],
+         [ '_month_day', '_at' ],
+         { truncate_to => 'minute' },
+       ],
+       [
+         { 0 => $RE{month}, 1 => $RE{monthday}, 2 => $RE{time_pm} },
+         [],
+         [],
+         [
+           [
+               1,
+             { 0 => [ $flag{month_name}, $flag{month_num} ] },
+           ],
+           [ 2 ],
+         ],
+         [ {}, { hours => 12 } ],
+         [ '_month_day', '_at' ],
+         { truncate_to => 'minute' },
+       ],
+    ],
     day_month_year_ago => [
       [ 'REGEXP', 'REGEXP', 'REGEXP', 'REGEXP', 'SCALAR' ],
       [
@@ -1168,8 +1414,8 @@ our (%init,
          [],
          [],
          [ [ 0 ] ],
-         [ { hours => 0 } ],
-         [ '_at' ],
+         [ {} ],
+         [ '_time' ],
          {
            prefer_future => true,
            truncate_to   => 'minute',
@@ -1181,7 +1427,7 @@ our (%init,
          [],
          [ [ 0 ] ],
          [ { hours => 12 } ],
-         [ '_at' ],
+         [ '_time' ],
          {
            prefer_future => true,
            truncate_to   => 'minute',
@@ -1195,7 +1441,7 @@ our (%init,
          [],
          [],
          [ [ 0 ] ],
-         [ { hours => 0 } ],
+         [ {} ],
          [ '_at' ],
          {
            prefer_future => true,
@@ -1227,8 +1473,8 @@ our (%init,
            ],
            [ 1 ],
          ],
-         [ {}, { hours => 0 } ],
-         [ '_weekday', '_time' ],
+         [ {}, {} ],
+         [ '_weekday', '_at' ],
          {
            prefer_future => true,
            truncate_to   => 'minute',
@@ -1245,7 +1491,7 @@ our (%init,
            [ 1 ],
          ],
          [ {}, { hours => 12 } ],
-         [ '_weekday', '_time' ],
+         [ '_weekday', '_at' ],
          {
            prefer_future => true,
            truncate_to   => 'minute',
@@ -1394,7 +1640,7 @@ our (%init,
          [],
          [],
          [ [ 0 ] ],
-         [ { hours => 0 } ],
+         [ {} ],
          [ '_daytime_in_the_variant' ],
          {},
        ],
@@ -1699,7 +1945,7 @@ our (%init,
            [ 5 ],
          ],
          [ { unit => 'month' }, {}, {} ],
-         [ '_ago_variant', '_weekday', '_time' ],
+         [ '_ago_variant', '_weekday', '_at' ],
          { truncate_to => 'minute' },
        ],
        [
@@ -1932,7 +2178,7 @@ our (%init,
            ],
            [ 1 ],
          ],
-         [ {}, { hours => 0 } ],
+         [ {}, {} ],
          [ '_weekday', '_daytime_in_the_variant' ],
          {},
        ],
@@ -2430,7 +2676,7 @@ our (%init,
              { 2 => [ $flag{before_after_from} ] },
            ],
          ],
-         [ { hour => 12 } ],
+         [ { hours => 12 } ],
          [ '_hourtime_variant' ],
          { truncate_to => 'hour' },
        ],
@@ -2444,7 +2690,7 @@ our (%init,
              { 2 => [ $flag{before_after_from} ] },
            ],
          ],
-         [ { hour => 0 } ],
+         [ {} ],
          [ '_hourtime_variant' ],
          { truncate_to => 'hour' },
        ],
@@ -2458,7 +2704,7 @@ our (%init,
              { 2 => [ $flag{before_after_from} ] },
            ],
          ],
-         [ { hour => 12 } ],
+         [ { hours => 12 } ],
          [ '_hourtime_variant' ],
          { truncate_to => 'hour' },
        ],
@@ -2472,12 +2718,99 @@ our (%init,
              { 2 => [ $flag{before_after_from} ] },
            ],
          ],
-         [ { hour => 0 } ],
+         [ {} ],
          [ '_hourtime_variant' ],
          { truncate_to => 'hour' },
        ],
     ],
-    day_at => [
+    day_am_pm => [
+       [ 'REGEXP', 'REGEXP' ],
+       [
+         { 0 => qr/^(yesterday)$/i, 1 => $RE{time_am} },
+         [],
+         [],
+         [
+           [
+             { 0 => [ $flag{yes_today_tom} ] },
+           ],
+           [ 1 ],
+         ],
+         [ { unit => 'day' }, {} ],
+         [ '_unit_variant', '_at' ],
+         { truncate_to => 'minute' },
+       ],
+       [
+         { 0 => qr/^(today)$/i, 1 => $RE{time_am} },
+         [],
+         [],
+         [
+           [
+             { 0 => [ $flag{yes_today_tom} ] },
+           ],
+           [ 1 ],
+         ],
+         [ { unit => 'day' }, {} ],
+         [ '_unit_variant', '_at' ],
+         { truncate_to => 'minute' },
+       ],
+       [
+         { 0 => qr/^(tomorrow)$/i, 1 => $RE{time_am} },
+         [],
+         [],
+         [
+           [
+             { 0 => [ $flag{yes_today_tom} ] },
+           ],
+           [ 1 ],
+         ],
+         [ { unit => 'day' }, {} ],
+         [ '_unit_variant', '_at' ],
+         { truncate_to => 'minute' },
+       ],
+       [
+         { 0 => qr/^(yesterday)$/i, 1 => $RE{time_pm} },
+         [],
+         [],
+         [
+           [
+             { 0 => [ $flag{yes_today_tom} ] },
+           ],
+           [ 1 ],
+         ],
+         [ { unit => 'day' }, { hours => 12 } ],
+         [ '_unit_variant', '_at' ],
+         { truncate_to => 'minute' },
+       ],
+       [
+         { 0 => qr/^(today)$/i, 1 => $RE{time_pm} },
+         [],
+         [],
+         [
+           [
+             { 0 => [ $flag{yes_today_tom} ] },
+           ],
+           [ 1 ],
+         ],
+         [ { unit => 'day' }, { hours => 12 } ],
+         [ '_unit_variant', '_at' ],
+         { truncate_to => 'minute' },
+       ],
+       [
+         { 0 => qr/^(tomorrow)$/i, 1 => $RE{time_pm} },
+         [],
+         [],
+         [
+           [
+             { 0 => [ $flag{yes_today_tom} ] },
+           ],
+           [ 1 ],
+         ],
+         [ { unit => 'day' }, { hours => 12 } ],
+         [ '_unit_variant', '_at' ],
+         { truncate_to => 'minute' },
+       ],
+    ],
+    day_time => [
        [ 'REGEXP', 'SCALAR', 'REGEXP' ],
        [
          { 0 => qr/^(yesterday)$/i, 1 => 'at', 2 => $RE{time_am} },
@@ -2490,7 +2823,7 @@ our (%init,
            [ 2 ],
          ],
          [ { unit => 'day' }, {} ],
-         [ '_unit_variant', '_time' ],
+         [ '_unit_variant', '_at' ],
          { truncate_to => 'minute' },
        ],
        [
@@ -2504,7 +2837,7 @@ our (%init,
            [ 2 ],
          ],
          [ { unit => 'day' }, {} ],
-         [ '_unit_variant', '_time' ],
+         [ '_unit_variant', '_at' ],
          { truncate_to => 'minute' },
        ],
        [
@@ -2518,6 +2851,93 @@ our (%init,
            [ 2 ],
          ],
          [ { unit => 'day' }, {} ],
+         [ '_unit_variant', '_at' ],
+         { truncate_to => 'minute' },
+       ],
+    ],
+    day_at_time => [
+       [ 'REGEXP', 'SCALAR', 'REGEXP', 'SCALAR' ],
+       [
+         { 0 => qr/^(yesterday)$/i, 1 => 'at', 2 => $RE{time}, 3 => 'am' },
+         [],
+         [],
+         [
+           [
+             { 0 => [ $flag{yes_today_tom} ] },
+           ],
+           [ 2 ],
+         ],
+         [ { unit => 'day' }, {} ],
+         [ '_unit_variant', '_time' ],
+         { truncate_to => 'minute' },
+       ],
+       [
+         { 0 => qr/^(today)$/i, 1 => 'at', 2 => $RE{time}, 3 => 'am' },
+         [],
+         [],
+         [
+           [
+             { 0 => [ $flag{yes_today_tom} ] },
+           ],
+           [ 2 ],
+         ],
+         [ { unit => 'day' }, {} ],
+         [ '_unit_variant', '_time' ],
+         { truncate_to => 'minute' },
+       ],
+       [
+         { 0 => qr/^(tomorrow)$/i, 1 => 'at', 2 => $RE{time}, 3 => 'am' },
+         [],
+         [],
+         [
+           [
+             { 0 => [ $flag{yes_today_tom} ] },
+           ],
+           [ 2 ],
+         ],
+         [ { unit => 'day' }, {} ],
+         [ '_unit_variant', '_time' ],
+         { truncate_to => 'minute' },
+       ],
+       [
+         { 0 => qr/^(yesterday)$/i, 1 => 'at', 2 => $RE{time}, 3 => 'pm' },
+         [],
+         [],
+         [
+           [
+             { 0 => [ $flag{yes_today_tom} ] },
+           ],
+           [ 2 ],
+         ],
+         [ { unit => 'day' }, { hours => 12 } ],
+         [ '_unit_variant', '_time' ],
+         { truncate_to => 'minute' },
+       ],
+       [
+         { 0 => qr/^(today)$/i, 1 => 'at', 2 => $RE{time}, 3 => 'pm' },
+         [],
+         [],
+         [
+           [
+             { 0 => [ $flag{yes_today_tom} ] },
+           ],
+           [ 2 ],
+         ],
+         [ { unit => 'day' }, { hours => 12 } ],
+         [ '_unit_variant', '_time' ],
+         { truncate_to => 'minute' },
+       ],
+       [
+         { 0 => qr/^(tomorrow)$/i, 1 => 'at', 2 => $RE{time}, 3 => 'pm' },
+         [],
+         [],
+         [
+           [
+             { 0 => [ $flag{yes_today_tom} ] },
+           ],
+           [ 2 ],
+         ],
+         [ { unit => 'day' }, { hours => 12 } ],
          [ '_unit_variant', '_time' ],
          { truncate_to => 'minute' },
        ],
@@ -2535,6 +2955,97 @@ our (%init,
            [ 2 ],
          ],
          [ {}, {} ],
+         [ '_weekday', '_at' ],
+         {
+           prefer_future => true,
+           truncate_to   => 'minute',
+         },
+       ],
+       [
+         { 0 => $RE{weekday}, 1 => 'at', 2 => $RE{time_pm} },
+         [],
+         [],
+         [
+           [
+             { 0 => [ $flag{weekday_name}, $flag{weekday_num} ] },
+           ],
+           [ 2 ],
+         ],
+         [ {}, { hours => 12 } ],
+         [ '_weekday', '_at' ],
+         {
+           prefer_future => true,
+           truncate_to   => 'minute',
+         },
+       ],
+    ],
+    weekday_at_am_pm => [
+       [ 'REGEXP', 'SCALAR', 'REGEXP', 'SCALAR' ],
+       [
+         { 0 => $RE{weekday}, 1 => 'at', 2 => $RE{time}, 3 => 'am' },
+         [],
+         [],
+         [
+           [
+             { 0 => [ $flag{weekday_name}, $flag{weekday_num} ] },
+           ],
+           [ 2 ],
+         ],
+         [ {}, {} ],
+         [ '_weekday', '_time' ],
+         {
+           prefer_future => true,
+           truncate_to   => 'minute',
+         },
+       ],
+       [
+         { 0 => $RE{weekday}, 1 => 'at', 2 => $RE{time}, 3 => 'pm' },
+         [],
+         [],
+         [
+           [
+             { 0 => [ $flag{weekday_name}, $flag{weekday_num} ] },
+           ],
+           [ 2 ],
+         ],
+         [ {}, { hours => 12 } ],
+         [ '_weekday', '_time' ],
+         {
+           prefer_future => true,
+           truncate_to   => 'minute',
+         },
+       ],
+    ],
+    weekday_am_pm => [
+       [ 'REGEXP', 'REGEXP', 'SCALAR' ],
+       [
+         { 0 => $RE{weekday}, 1 => $RE{time}, 2 => 'am' },
+         [],
+         [],
+         [
+           [
+             { 0 => [ $flag{weekday_name}, $flag{weekday_num} ] },
+           ],
+           [ 1 ],
+         ],
+         [ {}, {} ],
+         [ '_weekday', '_time' ],
+         {
+           prefer_future => true,
+           truncate_to   => 'minute',
+         },
+       ],
+       [
+         { 0 => $RE{weekday}, 1 => $RE{time}, 2 => 'pm' },
+         [],
+         [],
+         [
+           [
+             { 0 => [ $flag{weekday_name}, $flag{weekday_num} ] },
+           ],
+           [ 1 ],
+         ],
+         [ {}, { hours => 12 } ],
          [ '_weekday', '_time' ],
          {
            prefer_future => true,
@@ -2587,26 +3098,6 @@ our (%init,
          { truncate_to => 'minute' },
        ],
     ],
-    weekday_at_time_pm => [
-       [ 'REGEXP', 'SCALAR', 'REGEXP' ],
-       [
-         { 0 => $RE{weekday}, 1 => 'at', 2 => $RE{time_pm} },
-         [],
-         [],
-         [
-           [
-             { 0 => [ $flag{weekday_name}, $flag{weekday_num} ] },
-           ],
-           [ 2 ],
-         ],
-         [ {}, { hours => 12 } ],
-         [ '_weekday', '_at' ],
-         {
-           prefer_future => true,
-           truncate_to   => 'minute',
-         },
-       ],
-    ],
     day_month_year => [
        [ 'REGEXP', 'REGEXP', 'REGEXP' ],
        [
@@ -2692,6 +3183,20 @@ our (%init,
          [ '_final_weekday_in_month' ],
          { truncate_to => 'day' },
        ],
+       [
+         { 0 => 'last', 1 => $RE{weekday}, 2 => 'in', 3 => $RE{month} },
+         [],
+         [],
+         [
+           [
+             { 1 => [ $flag{weekday_name}, $flag{weekday_num} ] },
+             { 3 => [ $flag{month_name}, $flag{month_num} ] },
+           ],
+         ],
+         [ {} ],
+         [ '_final_weekday_in_month' ],
+         { truncate_to => 'day' },
+       ],
     ],
     for_count_unit => [
        [ 'SCALAR', 'REGEXP', 'REGEXP' ],
@@ -2773,6 +3278,67 @@ our (%init,
          {},
        ],
     ],
+    first_last_day_unit => [
+       [ 'SCALAR', 'SCALAR', 'SCALAR', 'REGEXP' ],
+       [
+         { 0 => 'first', 1 => 'day', 2 => 'of', 3 => $RE{month} },
+         [],
+         [],
+         [
+           [
+             { 3 => [ $flag{month_name}, $flag{month_num} ] },
+             { VALUE => 1 },
+           ],
+         ],
+         [ {} ],
+         [ '_first_last_day_unit' ],
+         { truncate_to => 'day' },
+       ],
+       [
+         { 0 => 'first', 1 => 'day', 2 => 'of', 3 => $RE{year} },
+         [],
+         [],
+         [
+           [
+               3,
+             { VALUE => 1 },
+             { VALUE => 1 },
+           ],
+         ],
+         [ {} ],
+         [ '_first_last_day_unit' ],
+         { truncate_to => 'day' },
+       ],
+       [
+         { 0 => 'last', 1 => 'day', 2 => 'of', 3 => $RE{month} },
+         [],
+         [],
+         [
+           [
+             { 3 => [ $flag{month_name}, $flag{month_num} ] },
+             { VALUE => undef },
+           ],
+         ],
+         [ {} ],
+         [ '_first_last_day_unit' ],
+         { truncate_to => 'day' },
+       ],
+       [
+         { 0 => 'last', 1 => 'day', 2 => 'of', 3 => $RE{year} },
+         [],
+         [],
+         [
+           [
+                3,
+              { VALUE => 12 },
+              { VALUE => undef },
+            ],
+         ],
+         [ {} ],
+         [ '_first_last_day_unit' ],
+         { truncate_to => 'day' },
+       ],
+    ],
 );
 
 1;
@@ -2880,10 +3446,26 @@ that the parser does not distinguish between lower/upper case):
  yesterday at 6:45pm
  today at 6:45pm
  tomorrow at 6:45pm
+ yesterday at 2:32 AM
+ today at 2:32 AM
+ tomorrow at 2:32 AM
+ yesterday at 2:32 PM
+ today at 2:32 PM
+ tomorrow at 2:32 PM
+ yesterday 2:32am
+ today 2:32am
+ tomorrow 2:32am
+ yesterday 2:32pm
+ today 2:32pm
+ tomorrow 2:32pm
  wednesday at 14:30
  wednesday at 02:30pm
  wednesday 14:30
  wednesday 02:30pm
+ friday 03:00 am
+ friday 03:00 pm
+ sunday at 05:00 am
+ sunday at 05:00 pm
  2nd monday
  100th day
  4th february
@@ -2959,9 +3541,21 @@ that the parser does not distinguish between lower/upper case):
  10 hours before midnight
  5 hours after noon
  5 hours after midnight
+ noon last friday
+ midnight last friday
+ noon this friday
+ midnight this friday
+ noon next friday
+ midnight next friday
  last friday at 20:00
  this friday at 20:00
  next friday at 20:00
+ 1am last friday
+ 1am this friday
+ 1am next friday
+ 1pm last friday
+ 1pm this friday
+ 1pm next friday
  yesterday at 13:00
  today at 13:00
  tomorrow at 13
@@ -2978,12 +3572,17 @@ that the parser does not distinguish between lower/upper case):
  fri 3 months ago at 5am
  wednesday 1 month ago at 8pm
  final thursday in april
+ last thursday in april
 
 =head2 Timespans
 
  Monday to Friday
  1 April to 31 August
  2009-03-10 9:00 to 11:00
+ first day of 2009 to last day of 2009
+ first day of may to last day of may
+ first to last day of 2008
+ first to last day of september
  for 4 seconds
  for 4 minutes
  for 4 hours
@@ -2998,6 +3597,8 @@ that the parser does not distinguish between lower/upper case):
  January 11
  11 January
  dec 25
+ feb 28 3am
+ feb 28 3pm
  may 27th
  2005
  march 1st 2009
